(* Common Subexpression Elimination optimization *)
open Sprog 

let cse_compare c1 c2 =
   match c1, c2 with 
     (op1, o11, o12), (op2, o21, o22) 
        when op1 = op2 && List.mem op1 [Eprog.Add; Eprog.Mul] ->
          if o11 = o21 && o12 = o22 ||
             o11 = o22 && o12 = o21 then 0 
          else compare (o11, o12) (o21, o22)  
    | _ -> compare c1 c2 

module StringMap = Map.Make (String)
module BinopMap = Map.Make 
    (struct type t = Eprog.binop * operand * operand 
            let compare = cse_compare                   
     end )

type cse_env = varname BinopMap.t
type subst_env = varname StringMap.t

let subst_oper (_, subst_env) = function 
     Lit n -> Lit n 
   | Var x -> 
       match StringMap.find_opt x subst_env with 
         None -> Var x 
       | Some y -> Var y

let cse_stmt ((cse_env, subst_env) as _env) stmt = 
  match stmt with 
    ValBinop (x, op, o1, o2) -> 
      let o1' = subst_oper _env o1 in 
      let o2' = subst_oper _env o2 in 
      ( match BinopMap.find_opt (op, o1', o2') cse_env with
          None -> Some (ValBinop (x, op, o1', o2')),
                    (BinopMap.add (op, o1', o2') x cse_env, subst_env )
        | Some y -> 
              None, (cse_env, StringMap.add x y subst_env))
  | Input _ -> Some stmt, _env  

module T = SimpleRewriter.Make ( struct 
      type env = cse_env * subst_env
      let empty = (BinopMap.empty, StringMap.empty)
      let rewrite_stmt = cse_stmt 
      let rewrite_oper = subst_oper
    end ) 

let cse = T.traverse