(* Constant Folding Optimization *)
open Sprog 

module StringMap = Map.Make (String)

type const_fold_env = operand StringMap.t

let subst_oper env = function 
   Lit n -> Lit n
 | Var x  -> 
      match StringMap.find_opt x env with 
        None -> Var x 
      | Some op -> op

let f_op = let open Eprog in function 
    Add -> ( + ) | Sub -> ( - ) | Mul -> ( * ) | Div -> ( / )

let const_fold_stmt env stmt = 
  let (|->) x op = None, StringMap.add x op env in 
  match stmt with 
  | ValBinop (x, op, Lit a, Lit b) ->  x |-> Lit ((f_op op) a b)
  | ValBinop (x, Add, Lit 0, Var y) -> x |-> Var y
  | ValBinop (x, Add, Var y, Lit 0) -> x |-> Var y
  | ValBinop (x, Sub, Var y, Lit 0) -> x |-> Var y
  | ValBinop (x, Mul, Lit 0, _    ) -> x |-> Lit 0
  | ValBinop (x, Mul, _,     Lit 0) -> x |-> Lit 0
  | ValBinop (x, Mul, Lit 1, Var y) -> x |-> Var y
  | ValBinop (x, Mul, Var y, Lit 1) -> x |-> Var y
  | ValBinop (x, Sub, Var a, Var b) when a = b -> x |-> Lit 0
  | ValBinop (y, op, oper1, oper2 ) -> 
      let _s = subst_oper env in
      Some (ValBinop (y, op, _s oper1, _s oper2)), env
  | Input _ -> Some stmt, env


(* we introduce the module SimpleRewriter because both constant 
   folding and common subexpression elimination are using 
   a similar iterative structure; for what happens in there
   see the commented out definition of const_fold_sprog2 
   below
   
   *)  
module T = SimpleRewriter.Make ( struct 
               type env = const_fold_env
               let empty = StringMap.empty 
               let rewrite_stmt = const_fold_stmt
               let rewrite_oper = subst_oper
               end )

let const_fold_sprog = T.traverse


(*   
let const_fold_sprog2 (stmts, e) = 
   let stmts', env = 
     List.fold_left 
        (fun (ii,env) stmt -> 
            match const_fold_stmt env stmt with 
               None, env' -> ii, env' 
            |  Some i, env' -> i::ii, env' 
        ) 
        ([], StringMap.empty )
        stmts in 
   (List.rev stmts', subst_oper env e) *)