open Sprog
let fresh_variable, current_counter_value = 
  let ctr = ref 0 in 
  ( fun x -> let c = !ctr in ctr := c + 1; x ^ (string_of_int c))
  , fun () -> !ctr

module StringMap = Map.Make (String)
let lookup = StringMap.find
let insert = StringMap.add 

type substenv = operand StringMap.t 

let rec tr_expr env (e:Eprog.expr) = 
  let tr = tr_expr env in 
  match e with 
    Int x -> [], Lit x 
  | BinOp (op, e1, e2) -> 
      let i1, o1 = tr e1 in  
      let i2, o2 = tr e2 in 
      let x = fresh_variable "t" in
      i1 @ i2 @ [ ValBinop (x, op, o1, o2)], Var x 
  | Var x ->
      [], (lookup x env)

let tr_stmt substenv = 
  let open Eprog in 
  function 
  Val (x,e) -> 
      let i, op = tr_expr substenv e in 
      i, insert x op substenv
| Input x -> 
      [ Sprog.Input x ], insert x (Sprog.Var x) substenv 

let eprog_to_sprog (stmts, e) = 
  let stmt_insts, substenv = 
    List.fold_left 
      ( fun (j,substenv) stmt  -> 
            let i, substenv' = tr_stmt substenv stmt in 
            j @ i, substenv' 
      )
      ([], StringMap.empty) stmts  in 
  let e_insts, e_op = tr_expr substenv e in 
  (stmt_insts @ e_insts), e_op

