(* eprogToSprogDestDriven.ml *)
open Sprog
let fresh_variable, current_counter_value = 
  let ctr = ref 0 in 
  ( fun x -> let c = !ctr in ctr := c + 1; x ^ (string_of_int c))
  , fun () -> !ctr

type t_env = (varname * operand) list

(* Simplification that assumes there is a way to avoid name clashing! *)
let rec tr_expr env (dest:varname option) (e:Eprog.expr) = 
  match e with 
    Int n -> [], Lit n
  | BinOp (op, e1, e2) -> 
      let i1, o1 = tr_expr env None e1 in  
      let i2, o2 = tr_expr env None e2 in 
      let x = 
          match dest with 
            None -> fresh_variable "$t"
          | Some y -> y in
      i1 @ i2 @ [ ValBinop (x, op, o1, o2)], Var x
  | Eprog.Var x -> [], 
          match List.assoc_opt x env with 
            None -> Var x 
          | Some o -> o 
      
      
let tr_stmt env = 
  let open Eprog in function 
    Val (x,e) -> 
        let i, o = tr_expr env (Some x) e 
        in (x, o)::env, i
  | Input x -> 
        (x, Sprog.Var x) :: env,
        [Sprog.Input x]

let eprog_to_sprog (stmts, e) = 
  let env', stmts' = List.fold_left_map tr_stmt [] stmts in
  let e_insts, e_op = tr_expr env' None e in 
  ((List.concat stmts') @ e_insts), e_op