(* sprog.ml *)
type binop = Eprog.binop
(* type binop = | Add | Sub | Mul | Div *)
type varname = Eprog.varname

type operand = Lit of int 
             | Var of varname 

type sstmt = 
   | ValBinop of varname * binop * operand * operand 
   | Input of varname

type sprog = sstmt list * operand  


(* Convert binop to string *)
let string_of_binop = let open Eprog in function
  | Add -> "+"
  | Sub -> "-"
  | Mul -> "*"
  | Div -> "/"

(* Convert operand to string *)
let string_of_operand = function
  | Lit i -> string_of_int i
  | Var v -> v

(* Convert estmt to string *)
let string_of_estmt = function
  | ValBinop (v, op, op1, op2) -> 
    v ^ " = " ^ (string_of_operand op1) ^ " " ^ (string_of_binop op) ^ " " ^ (string_of_operand op2)
  | Input v -> "input " ^ v

(* Convert eprog to string *)
let string_of_sprog (stmts, op) = 
   if stmts <> []
      then 
         let stmts_str = String.concat ";\n" (List.map string_of_estmt stmts) in
         stmts_str ^ ";\nreturn " ^ (string_of_operand op)
      else 
         "return " ^ (string_of_operand op)
